# ------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ------------------------------------------------------------------------------

from azdata.cli.core.util import FileUtil
from azdata.cli.core.logging import get_logger
from six import add_metaclass
from abc import (ABCMeta, abstractmethod)

import os

__all__ = ['WheelExtension']

logger = get_logger(__name__)


@add_metaclass(ABCMeta)
class BaseExtension(object):
    """
    Designed to be a low-level abstract foundation class from which other
    extension attribute classes can be derived.
    """

    def __init__(self, name, path):
        self._name = name
        self._path = path

    @property
    def name(self):
        return self._name

    @property
    def path(self):
        return self._path

    @abstractmethod
    def preview(self):
        """Sub-class should override and implement."""

    @abstractmethod
    def version(self):
        """Sub-class should override and implement."""

    @abstractmethod
    def metadata(self):
        """Sub-class should override and implement."""

    @abstractmethod
    def module_name(self):
        """Sub-class should override and implement."""

    @staticmethod
    def get_extension(name):
        """
        Returns wheel-based extension.
        """
        for ext in BaseExtension.get_extensions():
            if ext.name == name:
                return ext
        return {}

    @staticmethod
    def get_extensions():
        """
        Returns all wheel-based extensions.
        """
        from azdata.cli.core.configuration import Configuration
        #from azdata.cli.core.extensions import WheelExtension
        from glob import glob
        import os

        ext_dir = Configuration().EXTENSIONS_DIR
        exts = []

        if os.path.isdir(ext_dir):
            for ext_name in os.listdir(ext_dir):
                ext_path = os.path.join(ext_dir, ext_name)
                pattern = os.path.join(ext_path, '*.*-info')
                if os.path.isdir(ext_path) and glob(pattern):
                    exts.append(WheelExtension(ext_name, ext_path))
        return exts

    @staticmethod
    def get_extension_path(ext_name):
        """
        Returns the the extension file-path from name.
        :param ext_name: The extension name.
        :return: The extension file-path to its directory location.
        """
        from azdata.cli.core.configuration import Configuration
        return os.path.join(Configuration().EXTENSIONS_DIR, ext_name)

    @staticmethod
    def get_extension_name_from_wheel(source):
        """
        Parses a full-path `source` wheel filename to extract the section
        portion that represents the name of the module.
        :param source: The wheel file.
        :return: The extension name.
        """
        from azdata.cli.core.constants import WHEEL_INFO_RE

        whl_filename = os.path.basename(source)
        return WHEEL_INFO_RE(whl_filename).groupdict().get('name'). \
            replace('_', '-')


# ---------------------------------------------------------------------------- #
# ---------------------------------------------------------------------------- #
# ---------------------------------------------------------------------------- #


class WheelExtension(BaseExtension):
    """
    "A `BaseExtension` derivative class that also reads values from its
    .dist-info/METADATA.
    """
    def __init__(self, name, path):
        super(WheelExtension, self).__init__(name, path)

        self._version = None
        self._module_name = None
        self._metadata = None

    @property
    def preview(self):
        return self.metadata.get('isPreview', False)

    @property
    def version(self):
        from glob import glob
        from azdata.cli.core.util import FileUtil
        import os

        if self._version:
            return self._version

        dist_info = ''
        version_token = 'Version:'

        try:
            for dist_info_dir in glob(os.path.join(self.path, '*.*-info')):
                if self.name.replace('-', '_') in dist_info_dir:
                    dist_info = os.path.join(dist_info_dir, 'METADATA')
                    break

            if os.path.exists(dist_info):
                for line in FileUtil.read_text_file(dist_info).split('\n'):
                    if line.startswith(version_token):
                        self._version = line.split(version_token)[1].strip()
                        break
        except Exception:
            logger.warning('Failed to read version in "%s"', dist_info)

        return self._version

    @property
    def metadata(self):
        import json
        import traceback

        if self._metadata:
            return self._metadata

        self._metadata = {}

        ext_metadata_path = os.path.join(
            self.path, self.module_name, 'azdata_metadata.json')

        if os.path.exists(ext_metadata_path):
            try:
                self._metadata = FileUtil.read_json(ext_metadata_path)
            except json.decoder.JSONDecodeError as ex:
                logger.warn('Malformed metadata json: \n%s', ext_metadata_path)
                logger.debug('Unable to load extension metadata: %s',
                             traceback.format_exc())

        return self._metadata

    @property
    def module_name(self):
        from azdata.cli.core.configuration import Configuration

        if self._module_name:
            return self._module_name

        prefix = Configuration().EXTENSIONS_PREFIX
        pos_mods = [n for n in os.listdir(self.path) if n.startswith(prefix)
                    and os.path.isdir(os.path.join(self.path, n))]

        if len(pos_mods) != 1:
            raise AssertionError('Expected 1 module to load starting with '
                                 '"{}": got {}'.format(prefix, pos_mods))

        self._module_name = pos_mods[0]

        return self._module_name
